package service

import (
	"fmt"
	monitoringv1 "github.com/prometheus-operator/prometheus-operator/pkg/apis/monitoring/v1"
	"github.com/spf13/cast"
	"gitlab.wodcloud.com/smart-operation/so-operation-api/src/bean/entity"
	"gitlab.wodcloud.com/smart-operation/so-operation-api/src/bean/vo/response"
	"gitlab.wodcloud.com/smart-operation/so-operation-api/src/common/conf"
	"gitlab.wodcloud.com/smart-operation/so-operation-api/src/pkg/beagle/constant"
	"gitlab.wodcloud.com/smart-operation/so-operation-api/src/service/k8s"
	"go.uber.org/zap"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/util/intstr"
	"strings"
)

type PrometheusRuleSvc struct {
	User entity.SystemUserInfo
}

func (p *PrometheusRuleSvc) Create(data response.AlertRulesItem) (err error) {
	prometheusRuleName := k8s.GetPrometheusRuleName(data.Id)
	pr := monitoringv1.PrometheusRule{
		ObjectMeta: v1.ObjectMeta{
			Name:      prometheusRuleName,
			Namespace: k8s.Namespace,
			Labels:    k8s.AlertDefLabels,
		},
	}

	group := monitoringv1.RuleGroup{}
	groupInterval := monitoringv1.Duration(fmt.Sprintf("%d%s", data.CheckPeriod, "m"))
	group.Interval = &groupInterval
	ruleFor := monitoringv1.Duration(fmt.Sprintf("%d%s", data.Duration, data.DurationUnit))
	group.Name = k8s.GetPrometheusRuleGroupName(data.MetricConfigId, string(*group.Interval))
	for _, v := range data.AlertRange {
		item := fmt.Sprintf(`%s%s"%s"`, v.MetricLabel, v.Compare, v.Value) // http_requests_total{method="GET",pod="LeaseGrant"}
		data.Expr = strings.ReplaceAll(data.Expr, v.VariableName, item)
	}
	for _, v := range data.AlertCondition {
		rule := monitoringv1.Rule{
			Alert: data.MetricConfigName,
			For:   &ruleFor,
			Labels: map[string]string{
				"severity":        "warning",
				"risk_level":      cast.ToString(v.RiskLevel),
				"risk_level_name": constant.RiskLeveText(v.RiskLevel),
				"source":          "so-operation-api",
				"alert_rules_id":  data.MetricConfigId,
			},
			Annotations: map[string]string{
				"value":       "{{ $value }}",
				"summary":     fmt.Sprintf("分组名:%s, 检查周期:%s, 持续时间:%s", group.Name, string(groupInterval), string(ruleFor)),
				"description": "",
			},
		}

		var (
			condition int
			expr      string
		)

		if v.ThresholdsMin != nil {
			condition += 1
		}
		if v.ThresholdsMax != nil {
			condition += 2
		}

		switch condition {
		default:
			expr = data.Expr
		case 1:
			expr = fmt.Sprintf("%s <= %s", cast.ToString(v.ThresholdsMin), data.Expr)
		case 2:
			expr = fmt.Sprintf("%s <= %s", data.Expr, cast.ToString(v.ThresholdsMax))
		case 3:
			expr = fmt.Sprintf("%s <= %s <=%s", cast.ToString(v.ThresholdsMin), data.Expr, cast.ToString(v.ThresholdsMax))
		}

		rule.Expr = intstr.FromString(expr)
		group.Rules = append(group.Rules, rule)
	}
	pr.Spec.Groups = append(pr.Spec.Groups, group)
	header := map[string]string{"Authorization": "Bearer " + conf.Options.KubernetesToken}
	prSvc := k8s.PrometheusRule{Header: header}
	conf.Logger.Info("pr", zap.Any("pr", pr))
	err = prSvc.Create(&pr)
	return
}

func (p *PrometheusRuleSvc) Get(data response.AlertRulesItem) (obj *monitoringv1.PrometheusRule, exist bool, err error) {
	prometheusRuleName := k8s.GetPrometheusRuleName(data.Id)
	pr := monitoringv1.PrometheusRule{
		ObjectMeta: v1.ObjectMeta{
			Name:      prometheusRuleName,
			Namespace: k8s.Namespace,
			Labels:    k8s.AlertDefLabels,
		},
	}
	header := map[string]string{"Authorization": "Bearer " + conf.Options.KubernetesToken}
	prSvc := k8s.PrometheusRule{Header: header}
	conf.Logger.Info("pr", zap.Any("pr", pr))
	obj, err = prSvc.Get(pr.Namespace, pr.Name)
	if obj != nil && err == nil {
		exist = true
	}
	return
}

func (p *PrometheusRuleSvc) Delete(data response.AlertRulesItem) (err error) {
	prometheusRuleName := k8s.GetPrometheusRuleName(data.Id)
	pr := monitoringv1.PrometheusRule{
		ObjectMeta: v1.ObjectMeta{
			Name:      prometheusRuleName,
			Namespace: k8s.Namespace,
			Labels:    k8s.AlertDefLabels,
		},
	}

	header := map[string]string{"Authorization": "Bearer " + conf.Options.KubernetesToken}
	prSvc := k8s.PrometheusRule{Header: header}
	conf.Logger.Info("pr", zap.Any("pr", pr))
	err = prSvc.Delete(pr.Namespace, pr.Name)
	return
}
